/*
 * Integer-executing version of a portion of rsc_c(), where coefficient
 * computation is intended to be offline and only the execution of the
 * actual PSV and Sd computation is in the instruments, this portion
 * performed with integer arithmatic.
 *
 * NOTA BENE -- Computation of Sa (needed for ShakeMap) can proceed
 * similarly, but has not been implemented here.  For many data
 * sources, the accelerogram will have to be changed to 32-bit
 * integers to avoid overflow.
 *
 * Computes ("structural") response spectra by the method of:
 *
 *    Nigam, N. C., and P. C. Jennings, Calculation of response spectra
 *    from strong-motion earthquake records, Bull. Seis. Soc. Am., 59,
 *    909922, 1969.
 *
 * This is a direct transcription of their equations, assuming a constant
 * digitizing interval, dt (i.e., sps samples per second) and offline
 * computation of shifted-integer coefficients and integer trace.
 *
 * RETURNS:
 *	rsc_c():  coefficients used by rsc_x().  Run on offline computer
 *            and load result into a file to be read into rsc_x().
 *	rsc_x():  pseudo-velocity response spectra and Sd computation.
 *            To be incorporated into instrument firmware.
 *
 * AUTHOR:  John R. Evans
 *		USGS, MS-977
 *		Menlo Park CA 94025
 *		evans@andreas.wr.usgs.gov
 *
 * USAGE:  psv_tst2006 damp sps gal_scale Gal2Int_scale [-j] < seismogram > result
 *
 * where "-j" gives results for a small set of periods recommended as
 * the minimum by Bill Joyner (the default is a larger set of periods
 * taken from Linda Seekins' abortive CD of processed strong-motion
 * data); "damp" is the damping, in percent; "sps" is the sampling
 * rate (e.g., 200 Hz); and "gal_scale" relates the input seismogram
 * values to cm/s/s as in:
 *     cm/s/s = input * gal_scale
 *
 * The input is scaled by this value so that output is in cm, cm/s,
 * and cm/s/s.  Defined this way, for example, "gal_scale" would be in
 * "cm/s/s per count" if the input is raw ADC output.  Of course,
 * if the seismogram is already in cm/s/s, set gal_scale = 1.0.
 *
 * Gal2Int_scale scales from cm/s/s to a (possibly shifted) integer
 * value, as in
 *     <[scaled] counts> = round(input * gal_scale * Gal2Int_scale + 0.5)
 *
 * Modified from rsc().  Finished 01 Dec 1997.  Major pre-releas
 * modifications Nov 2006.  The contained main(), used to compile
 * test routine psv_test2006() is derreived from rsc_real() and
 * rsc_test2006().
 *
 */

#define A21_FIRST	(1)
#define A22_FIRST	(2)

#define streq	!strcmp
#define NPTS	(100000)		/* Largest seismogram length */

typedef struct {

	int			comp_order;		/* Computation order for ai21 and ai22. */

	/* Coefficients (shifted up) */
	long int	ai11, ai12;
	long int	bi11, bi12;

	long int	ai21, ai22;
	long int	bi21, bi22;

	/* Coefficient shifts */
	long int	ai11_sft;		/* Shift remaining in Sd. */
	long int	ai12_sft;
	long int	bi11_sft;
	long int	bi12_sft;

	long int	ai21_sft;
	long int	ai22_sft;
	long int	bi21_sft;
	long int	bi22_sft;

	long int	ai2x_sft;		/* Shift remaining in Sv (ai21_sft or ai22_sft) */
	long int	ai2x_rnd;

	/* Rounding constants (equal 0.5 of LSB-after-shifting) */

	long int	ai11_xo_rnd;
	long int	ai11_fudge_rnd;

	long int	ai11_rnd;
	long int	ai12_rnd;
	long int	bi11_rnd;
	long int	bi12_rnd;

	long int	ai21_rnd;
	long int	ai22_rnd;
	long int	bi21_rnd;
	long int	bi22_rnd;

	/* Natural-frequency and sampling interval */
	long int	period;			/* Natural period of oscillator [ms]			*/
	long int	ww;				/* Natural freq. of oscillator [shifted Hz]		*/
	long int	ww_sft;			/* Shift for ww [bits]							*/
	long int	ww_fudge_rnd;	/* Rounding for output							*/
	long int	dt;				/* Digitizing interval [us]						*/

} rsc_cf;

typedef struct {
	long int	dd;				/* Maximum displacement, Sd [shifted counts]	*/
	long int	psv;			/* Maximum "pseudovelocity", ww*Sd [ "  " ]		*/
	int			di;				/* Sample number where dd and psv happen		*/
} rsc_rslt;

/* *************************************************************************
 * Test of rsc_c() and rsc_x().  Run a real accelerogram through at a
 * series of building periods.  Output response spectrum.
 *
 * USAGE:

make -f Makefile.psv_c
/bin/rm -f core out.all ORIGINAL NEW
sun4/psv_c > out.all

echo "out.original:" ; cat out.original ; echo "out.all:" ; cat out.all
nawk '{print $1, $4, $8, $11}' out.original | sed '1d' > ORIGINAL ; rcp2s ORIGINAL ; echo "ORIGINAL:" ; cat ORIGINAL
nawk '{print $1, $4, $7, $10}' out.all | sed '1d' > NEW ; rcp2s NEW ; echo "NEW:" ; cat NEW

 *
 * ************************************************************************* */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "mathconst.h"

/* FUDGE FACTORS (bits) */
static long int psv_fudge = (long int)4;
static long int xo_fudge  = (long int)3;
static long int xo_rnd    = (long int)4;

#define ABS(a)			( ((long int)(a) >= (long int)0) ? (a) : (-(a)) )
#define MIN(a, b)		( (a) < (b) ? (a) : (b) )
#define MAX(a, b)		( (a) > (b) ? (a) : (b) )

	rsc_cf		rsc_c();				/* Function defined below						*/
	rsc_rslt	rsc_x();				/* Function defined below						*/
	size_t		strlen();

int
main(argc, argv)
	int			argc;
	char		*argv[];
{
	int			iPer;					/* Pointer to structural period					*/

	double		hh;						/* Damping fracton (typically 0.05, i.e., 5%)	*/
	double		ww;						/* Natural frequency of the oscillator
										   (2*PI/period) [Hz]							*/
	double		dt;						/* Digitizing interval [s]						*/

	double		aa_buff;				/* Input buffer for seismogram					*/
	short int	aa[NPTS];				/* Input accelerogram [counts]					*/
	int			nn;						/* Number of points in accelerogram				*/

	rsc_cf		cf;
	rsc_rslt	rslts;

	double		exp2(), scaled, scalepsv;
	double		damp, sps, gal_scale, Gal2Int_scale;
	char		inbuf[128];

#if 0
	char	*fname;
	FILE	*fid;
#endif

	/* Response spectrum periods */

	/* Joyner/Boore "ten" (plus one more for ShakeMap) */
#define NPERSj	(11)
	double		bperj[NPERSj];

	/* From Linda Seekins' CD (unpublished) */
#define NPERS	(96)
	double		bper[NPERS];
	
	double		*struc_per;
	int			nper;


/* ***************************************************************************** */
	bperj[0]  = 0.1;
	bperj[1]  = 0.15;
	bperj[2]  = 0.2;
	bperj[3]  = 0.3;
	bperj[4]  = 0.4;
	bperj[5]  = 0.5;
	bperj[6]  = 0.75;
	bperj[7]  = 1.0;
	bperj[8]  = 1.5;
	bperj[9]  = 2.0;
	bperj[10] = 3.0;
/* ***************************************************************************** */
	bper[ 1 - 1] = 0.02;
	bper[ 2 - 1] = 0.02;
	bper[ 3 - 1] = 0.05;
	bper[ 4 - 1] = 0.10;
	bper[ 5 - 1] = 0.20;
	bper[ 6 - 1] = 0.04;
	bper[ 7 - 1] = 0.04;
	bper[ 8 - 1] = 0.04;
	bper[ 9 - 1] = 0.05;
	bper[10 - 1] = 0.05;
	bper[11 - 1] = 0.05;
	bper[12 - 1] = 0.06;
	bper[13 - 1] = 0.06;
	bper[14 - 1] = 0.07;
	bper[15 - 1] = 0.07;
	bper[16 - 1] = 0.07;
	bper[17 - 1] = 0.08;
	bper[18 - 1] = 0.09;
	bper[19 - 1] = 0.09;
	bper[20 - 1] = 0.10;
	bper[21 - 1] = 0.10;
	bper[22 - 1] = 0.11;
	bper[23 - 1] = 0.12;
	bper[24 - 1] = 0.13;
	bper[25 - 1] = 0.14;
	bper[26 - 1] = 0.15;
	bper[27 - 1] = 0.16;
	bper[28 - 1] = 0.17;
	bper[29 - 1] = 0.18;
	bper[30 - 1] = 0.19;
	bper[31 - 1] = 0.20;
	bper[32 - 1] = 0.22;
	bper[33 - 1] = 0.24;
	bper[34 - 1] = 0.26;
	bper[35 - 1] = 0.28;
	bper[36 - 1] = 0.30;
	bper[37 - 1] = 0.32;
	bper[38 - 1] = 0.34;
	bper[39 - 1] = 0.36;
	bper[40 - 1] = 0.38;
	bper[41 - 1] = 0.40;
	bper[42 - 1] = 0.42;
	bper[43 - 1] = 0.44;
	bper[44 - 1] = 0.46;
	bper[45 - 1] = 0.48;
	bper[46 - 1] = 0.50;
	bper[47 - 1] = 0.55;
	bper[48 - 1] = 0.60;
	bper[49 - 1] = 0.65;
	bper[50 - 1] = 0.70;
	bper[51 - 1] = 0.75;
	bper[52 - 1] = 0.80;
	bper[53 - 1] = 0.85;
	bper[54 - 1] = 0.90;
	bper[55 - 1] = 0.95;
	bper[56 - 1] = 1.00;
	bper[57 - 1] = 1.10;
	bper[58 - 1] = 1.20;
	bper[59 - 1] = 1.30;
	bper[60 - 1] = 1.40;
	bper[61 - 1] = 1.50;
	bper[62 - 1] = 1.60;
	bper[63 - 1] = 1.70;
	bper[64 - 1] = 1.80;
	bper[65 - 1] = 1.90;
	bper[66 - 1] = 2.00;
	bper[67 - 1] = 2.20;
	bper[68 - 1] = 2.40;
	bper[69 - 1] = 2.60;
	bper[70 - 1] = 2.80;
	bper[71 - 1] = 3.00;
	bper[72 - 1] = 3.20;
	bper[73 - 1] = 3.40;
	bper[74 - 1] = 3.60;
	bper[75 - 1] = 3.80;
	bper[76 - 1] = 4.00;
	bper[77 - 1] = 4.20;
	bper[78 - 1] = 4.40;
	bper[79 - 1] = 4.60;
	bper[80 - 1] = 4.80;
	bper[81 - 1] = 5.00;
	bper[82 - 1] = 5.50;
	bper[83 - 1] = 6.00;
	bper[84 - 1] = 6.50;
	bper[85 - 1] = 7.00;
	bper[86 - 1] = 7.50;
	bper[87 - 1] = 8.00;
	bper[88 - 1] = 8.50;
	bper[89 - 1] = 9.00;
	bper[90 - 1] = 9.50;
	bper[91 - 1] =10.00;
	bper[92 - 1] =11.00;
	bper[93 - 1] =12.00;
	bper[94 - 1] =13.00;
	bper[95 - 1] =14.00;
	bper[96 - 1] =15.00;
/* ***************************************************************************** */

	/* Get arguments */
	if (argc != 5 && argc != 6) {
		exit(1);
	}
		
	damp          = atof(argv[1]);
	sps           = atof(argv[2]);
	gal_scale     = atof(argv[3]);
	Gal2Int_scale = atof(argv[4]);	/* Keep Gal2Int_scale <= 32727/max(|data|)
									 * [count/(cm/s/s)] for short integers and
									 * <= ((2^31) - 1)/max(|data|) for long
									 * integers.  Overflows are detected and
									 * produce an error message at exection.	*/
									 
	/* printf("Debug:  damp = %lf; sps = %lf; gal_scale = %lf; Gal2Int_scale = %lf\n", damp, sps, gal_scale, Gal2Int_scale); */

	if (argc == 6 && streq("-j", argv[5])) {
		nper		= NPERSj;
		struc_per	= bperj;
	}
	else {
		nper		= NPERS;
		struc_per	= bper;
	}	

	/* Initialize */
	hh = damp / (double)100.;
	dt = (double)1.0 / sps;

	/* read accelerogram */
	nn = 0;
	while (NULL != fgets(inbuf, (int)128, stdin)) {
	
		if ((int)strlen(inbuf) > (int)127) {
			printf("ERROR:  Found string of length %d ('%s')", strlen(inbuf), inbuf);
			exit(2);
		}
	
		if (1 != sscanf(inbuf, "%lf", &(aa_buff))) {
			exit(3);
		}

		aa_buff = gal_scale * aa_buff * Gal2Int_scale + (double)0.5;

		if (fabs(aa_buff) >= (double)32767.5) {
			printf("\nInput accelerogram overflows short int at sample %d.\n", nn);
			printf("  Reduce Gal2Int_scale or command-line gal_scale factor:  '%s'\n\n", inbuf);
			exit(4);
		}
		
		if (nn > NPTS) {
			printf("Trace length exceeds NPTS = %d\n", NPTS);
		}
		
		aa[nn] = (short int)(aa_buff);							/* Trim to +/-32,767 */
		nn++;
	}


	/* Run rsc_x() across accelerograms */
	for (iPer = 0 ; iPer < nper ; iPer++) {
	
		ww = (double)2.0 * PI / struc_per[iPer];

		/* compute coefficients */
		cf = rsc_c(hh, ww, struc_per[iPer], dt);

		/* compute PSV for that period */
		rslts = rsc_x(cf, aa, nn);

		/* Deshift by Gal2Int_scale to get back to GGS units */
		scaled   = exp2( -((double)(cf.ai11_sft           )) ) / Gal2Int_scale;
		scalepsv = exp2( -((double)(cf.ww_sft + psv_fudge )) ) / Gal2Int_scale;

		(void)printf("%lf s, Sd= %0.10lf cm, psv= %0.10lf cm/s @ %d\n",
			struc_per[iPer],
			(double)(rslts.dd  + cf.ai11_rnd    ) * scaled,
			(double)(rslts.psv + cf.ww_fudge_rnd) * scalepsv,
			rslts.di);
	}

	exit(0);
}

/* *************************************************************************
 *
 * Compute coefficients used in PSV calculation and prepare them for
 * integer computations by scaling tham up by 2^m, that is:
 *
 * Compute integer scale factors m such that (coefficient * 2^m) is the
 * largest possible long integer smaller than 2^15.  Used to retain
 * computational accuracy by shifting each part up to the largest integer
 * that will not create an overflow when multiplied by a 16-bit signed-
 * integer sample (with two such results summed).  In other words, choose
 * m so that |coefficient| * 2^m <= 2^15 - 1.
 *
 * rsc_x() sums from the most-shifted to the least shifted term.  The
 * shift value stored for each coefficient is the difference from that
 * term's shift to the next term's along the sequence of computation--
 * ORDER MATTERS and MUST match that in rsc_x().
 *
 * Similarly the structure's natural frequency, ww, is shifted by ww_sft
 * (to a 15-bit number), the corresponding period is scaled up to integer
 * milliseconds, and the sample interval is scaled up to microseconds.
 *
 * ************************************************************************* */

#define LOG2_32767	(log2( (double)(32767.) ))		/* log2(2^15 - 1) 		*/

/* Compute rounding constants (0.5 of LSB-after-shifting) (exp2() ensures >= 0). */
#define ROUNDER(m)	((long)(exp2((double)(m) - (double)(1.0)) + (double)(1.e-6)))
/* ()s:  		1______2    3___________   _____________3   _______________21 */

rsc_cf
rsc_c(hh, ww, period, dt)
	double		hh;					/* Damping fracton (typically 0.05, i.e., 5%)	*/
	double		ww;					/* Natural frequency of the oscillator
									   (2*PI/period) [Hz]							*/
	double		period;				/* Natural period of the oscillator [s]			*/
	double		dt;					/* Digitizing interval [s]						*/
{
	double		a11, a12, a21, a22;	/* Matrix A coefficients						*/
	double		b11, b12, b21, b22;	/* Matrix B coefficients						*/
	rsc_cf		cf;					/* Result (shifted coefficients etc.)			*/

	double	sin(), cos(), sqrt(), exp(), fabs(), log2(), exp2();

	/* Often-repeated portions of A and B coefficients (pardon the names!) */
	double cosbit, sinbit, hratio, expbit, blue, yellow;

	/* ... and even smaller portions. */
	double rootbit, wrootbit, w2, w3;

	/* ******************************************************************** */
	/* ** Compute coefficients of A and B matrices of Nigam and Jennings ** */

	/* Fist the small portions ... */

	w2 = ww * ww;
	w3 = ww * w2;

	rootbit = sqrt(1.0 - hh * hh);
	wrootbit = ww * rootbit;

	cosbit = cos(wrootbit * dt);
	sinbit = sin(wrootbit * dt);
	hratio = hh / rootbit;

	expbit = exp(-hh * ww * dt);
	blue = 2.0 * hh / (w3 * dt);
	yellow = (2.0 * hh * hh - 1.0) / (w2 * dt);

	/* ... then the coefficients per se. */

	a11 = expbit * (hratio * sinbit + cosbit);
	a12 = expbit * sinbit / wrootbit;
	a21 = -ww * expbit * sinbit / rootbit;
	a22 = expbit * (cosbit - hratio * sinbit);					/* Eqn. (6d) */

	b11 = expbit * ((yellow + hh / ww) * (sinbit / wrootbit)
		+ (blue + 1.0 / w2) * cosbit) - blue;
	b12 = -expbit * (yellow * sinbit / wrootbit + blue * cosbit)
		- (1.0 / w2) + blue;
	b21 = expbit * ((yellow + hh / ww) * (cosbit - hratio * sinbit)
		- (blue + 1.0 / w2) * (wrootbit * sinbit + hh * ww * cosbit))
		+ 1.0 / (w2 * dt);
	b22 = -expbit * (yellow * (cosbit - hratio * sinbit)
		- blue * (wrootbit * sinbit + hh * ww * cosbit))
		- 1.0 / (w2 * dt);										/* Eqn. (6e) */


	/* ******************* Scale and convert coefficients ******************* */

	/* **************************** Displacement **************************** */

	/*
 	*   Scale factors:  |coeff| * 2^m <= 2^15 - 1
 	*   Hence:  m <= log2(2^15 - 1) - log2(|coeff|)
 	*/
	cf.bi12_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(b12)) ));
	cf.bi11_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(b11)) ));
	cf.ai12_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(a12)) ));
	cf.ai11_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(a11)) ));

	/* Do the scaling */
	cf.bi12 = (long int)(b12 * exp2((double)(cf.bi12_sft)) + (double)0.5);
	cf.bi11 = (long int)(b11 * exp2((double)(cf.bi11_sft)) + (double)0.5);
	cf.ai12 = (long int)(a12 * exp2((double)(cf.ai12_sft)) + (double)0.5);
	cf.ai11 = (long int)(a11 * exp2((double)(cf.ai11_sft)) + (double)0.5);

	/* Difference cascade */
	cf.bi12_sft = MAX((long int)0, cf.bi12_sft - cf.bi11_sft);
	cf.bi11_sft = MAX((long int)0, cf.bi11_sft - cf.ai12_sft);
	cf.ai12_sft = MAX((long int)0, cf.ai12_sft - cf.ai11_sft);
/*	cf.ai11_sft = cf.ai11_sft; */
	if (cf.ai11_sft < xo_fudge) {
		fprintf(stderr, "Programming error:  xo_fudge => ai11_sft\n");
		exit(5);
	}

	/* Rounding constants */
	cf.bi12_rnd = ROUNDER(cf.bi12_sft);
	cf.bi11_rnd = ROUNDER(cf.bi11_sft);
	cf.ai12_rnd = ROUNDER(cf.ai12_sft);
	cf.ai11_rnd = ROUNDER(cf.ai11_sft);

	cf.ai11_xo_rnd = ROUNDER(cf.ai11_sft - xo_fudge);

	/* ***************************** Velocity ****************************** */

	if (fabs(a21) < fabs(a22)) {
		cf.comp_order = A21_FIRST;
	}
	else {
		cf.comp_order = A22_FIRST;
	}

	/*
 	*   Scale factors:  |coeff| * 2^m <= 2^15 - 1
 	*   Hence:  m <= log2(2^15 - 1) - log2(|coeff|)
 	*/
	cf.bi21_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(b21)) ));
	cf.bi22_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(b22)) ));
	cf.ai22_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(a22)) ));
	cf.ai21_sft = MAX((long int)0, (long int)( LOG2_32767 - log2(fabs(a21)) ));

	/* Do the scaling */
	cf.bi22 = (long int)(b22 * exp2((double)(cf.bi22_sft)) + (double)0.5);
	cf.bi21 = (long int)(b21 * exp2((double)(cf.bi21_sft)) + (double)0.5);
	cf.ai22 = (long int)(a22 * exp2((double)(cf.ai22_sft)) + (double)0.5);
	cf.ai21 = (long int)(a21 * exp2((double)(cf.ai21_sft)) + (double)0.5);

	/* Difference cascade. */
	if (cf.comp_order == A22_FIRST) {
		cf.bi22_sft = MAX((long int)0, cf.bi22_sft - cf.bi21_sft);
		cf.bi21_sft = MAX((long int)0, cf.bi21_sft - cf.ai22_sft);
		cf.ai22_sft = MAX((long int)0, cf.ai22_sft - cf.ai21_sft);
		cf.ai2x_sft = cf.ai21_sft;
	}
	else {
		cf.bi22_sft = MAX((long int)0, cf.bi22_sft - cf.bi21_sft);
		cf.bi21_sft = MAX((long int)0, cf.bi21_sft - cf.ai21_sft);
		cf.ai21_sft = MAX((long int)0, cf.ai21_sft - cf.ai22_sft);
		cf.ai2x_sft = cf.ai22_sft;
	}

	/* Rounding constants */
	cf.bi22_rnd = ROUNDER(cf.bi22_sft);
	cf.bi21_rnd = ROUNDER(cf.bi21_sft);
	cf.ai22_rnd = ROUNDER(cf.ai22_sft);
	cf.ai21_rnd = ROUNDER(cf.ai21_sft);

	cf.ai2x_rnd = ROUNDER(cf.ai2x_sft);

	/* *************** Natural frequency and sample interval *************** */

	cf.period = (long int)((double)1000. * period + (double)0.5);

	cf.ww_sft = (long int)( LOG2_32767 - log2(ww) );

	/* Shift some precision in favor of Sd (aka "rslts.dd") */
	psv_fudge = MIN(cf.ai11_sft, MIN(psv_fudge, cf.ww_sft));
	cf.ww_sft -= psv_fudge;

	cf.ww_fudge_rnd = ROUNDER(cf.ww_sft + psv_fudge);
	cf.ai11_fudge_rnd = ROUNDER(cf.ai11_sft - psv_fudge);

	/* Scale ww and dt */
	cf.ww = (long int)(ww * exp2((double)(cf.ww_sft)) + (double)0.5);
	cf.dt = (long int)(dt * (double)1000000. + (double)0.5);	/* --> us */

	/* ***************************** Finished ***************************** */
	return (cf);
}

/* *************************************************************************
 * Compute PSV response spectra, given the coefficients.  Results are left
 * shifted up by "ai11_sft" for Sd (rslts.dd) and by "psv_fudge + ww_sft"
 * for the PSV value (rslts.psv).
 *
 * Computation is all long int.  Computation proceeds from the smallest-
 * magnitude to the largest-magnitude coefficient, rounding and down
 * shifting by the minimum amount required to avoid overflow at the next
 * step.  The computational order therefore is (left to right):
 *
 * Sd:  ((((bi12) + bi11) + ai12) + ai11)
 *
 * Sv:  ((((bi22) + bi21) + ai22) + ai21), if |ai22| < |ai21| (period<~0.75 s)
 * 	((((bi22) + bi21) + ai21) + ai22), if |ai22| >= |ai21|
 *
 * Using >> in place of division is faster but causes a uniform round-
 * toward-negative-infinity, which biases the result.  First adding 1/2
 * the "denominator" compensates and causes a uniform round-to-nearest-
 * integer (although with +/-0.5 rounding toward positive infinity, hence
 * a slight remaining bias).
 * ************************************************************************* */

rsc_rslt
rsc_x(cf, aa, nn)
	rsc_cf		cf;				/* Matrix coefficients etc. from rsc_c()	*/
	short int	*aa;			/* Pointer to accelerogram [counts]			*/
	int			nn;				/* Number of samples in accelerogram		*/
{
	rsc_rslt		rslts;

static long int		xo;			/* Displacement (Sd) at time ii-1			*/
	long int		xn;			/* Displacement (Sd) at time ii				*/
static long int 	vo;			/* Velocity (Sv) at time ii-1				*/
	long int		vn;			/* Velocity (Sv) at time ii					*/
static long int		ao;			/* Acceleration (input) at time ii-1		*/
	long int		an;			/* Acceleration (input) at time ii			*/
	int				ii;			/* Time pointer								*/

	/* Initialize */
	rslts.dd = (long int)(0);
	rslts.psv = (long int)(0);
	rslts.di = -1;

	xo = (long)0;
	xn = (long)0;
	vo = (long)0;
	vn = (long)0;

	/* Run along the accelerogram (beginning at SECOND point) */

	for (ii = 1 ; ii < nn ; (
			ii++,
			(xo = ((xn + cf.ai11_xo_rnd) >> (cf.ai11_sft - xo_fudge))),
			(vo = ((vn + cf.ai2x_rnd) >> cf.ai2x_sft))
		)) {

		ao = (long int)(aa[ii - 1]);
		an = (long int)(aa[ii]);

		/*
 		* Eqn. (6a-c):
 		*/

		/* ***** Displacement ***** */
		xn = ((((((
				(cf.bi12  * an + cf.bi12_rnd)   >> cf.bi12_sft ) +
				(cf.bi11  * ao + cf.bi11_rnd) ) >> cf.bi11_sft ) +
				(cf.ai12  * vo + cf.ai12_rnd) ) >> cf.ai12_sft ) +
				((cf.ai11 * xo + xo_rnd)        >> xo_fudge) );
				/*             + cf.ai11_rnd) ) >> cf.ai11_sft ) */

		/* ***** Velocity ***** */
		if (cf.comp_order == A22_FIRST) {
			vn = ((((((
				(cf.bi22 * an  + cf.bi22_rnd)   >> cf.bi22_sft ) +
				(cf.bi21 * ao  + cf.bi21_rnd) ) >> cf.bi21_sft ) +
				(cf.ai22 * vo  + cf.ai22_rnd) ) >> cf.ai22_sft ) +
				((cf.ai21 * xo + xo_rnd)        >> xo_fudge) );
 				/*             + cf.ai21_rnd) ) >> cf.ai21_sft ) */
		}
		else {
			vn = ((((((
				(cf.bi22    * an + cf.bi22_rnd)   >> cf.bi22_sft ) +
				(cf.bi21    * ao + cf.bi21_rnd) ) >> cf.bi21_sft ) +
				( ((cf.ai21 * xo + xo_rnd)        >> xo_fudge)
				                 + cf.ai21_rnd) ) >> cf.ai21_sft ) +
				(cf.ai22    * vo) );
 				/*               + cf.ai22_rnd) ) >> cf.ai22_sft ) */
		}

		if (ABS(xn) > rslts.dd) {
			rslts.dd = ABS(xn);
			rslts.psv = cf.ww *									/* Eqn. (8a) */
				((rslts.dd + cf.ai11_fudge_rnd)
					>> (cf.ai11_sft - psv_fudge));
			rslts.di = ii;
		}
	}

	return(rslts);
}
