/*
 * Run a seismogram through rsc() at a series of building periods
 * for any damping.  Output the response spectra (Sd, PSV, Sv, Sa)
 * as a series of MatLab vectors, plus a vector of periods (per).
 *
 * USAGE:  rsc_tst2006 damp sps scale [-j] < seismogram > result
 *
 * where "-j" gives results for a small set of periods recommended as
 * the minimum by Bill Joyner (the default is a larger set of periods
 * taken from Linda Seekins' abortive CD of processed strong-motion
 * data); "damp" is the damping, in percent; "sps" is the sampling
 * rate (e.g., 200); and "scale" related the input seismogram values
 * to cm/s/s as in:
 *     cm/s/s = input * scale
 * The input is scaled by this value so that output is in cm, cm/s,
 * and cm/s/s.  Defined this way, for example, "scale" would be in
 * "cm/s/s per count" if the input is raw ADC output.  (This is the
 * inverse of the definition used in ft(), be forwarned.)  Of course,
 * if the seismogram is already in cm/s/s, set scale = 1.0.
 *
 * BUGS:  Dumb about allocation.  Just limits the array to a maximum
 * of NPTS points.  Done only for programming speed and should be fixed.
 *
 * NOTA BENE:  rsc() is the authoritative code for generating test
 * values for psv_x().
 *
 * JREvans, USGS, 14 June 2001.  Modified Nov 2006 for release.
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "mathconst.h"
#include "rsc_result.h"

#define streq	!strcmp
#define NPTS	100000	/* Largest seismogram length */

	rsc_result	rsc();					/* rsc.c			*/
	void		write_vec();			/* below			*/

int
main(argc, argv)
	int			argc;
	char		*argv[];
{
	int			ii;
/*	int			jj; */

	double		damp, sps, scale;
	char		inbuf[128];

	double		hh;						/* damping fracton (typically 0.05, i.e., 5%)	*/
	double		ww;						/* natural frequency of the oscillator			*
										 * (2*PI/period)                         [Hz]	*/
	double		dt;						/* digitizing interval                   [s]	*/
	double		aa[NPTS];				/* the input accelerogram            [cm/s/s]	*/
	int			nn;						/* number of sample points in accelerogram		*/

	rsc_result	result;

	FILE		*fopen();
/*	FILE		*fid; /

/* ************************* response spectral periods ************************* */
	int			nper;
	double		*per;

	/* Joyner/Boore "ten" (plus one more for ShakeMap) */
#define NPERSj	(11)
	double		bperj[NPERSj];

	/* From Linda Seekins' CD (unpublished) */
#define NPERS	(96)
	double		bper[NPERS];

	double		Sd[NPERS];
	double		PSV[NPERS];
	double		Sv[NPERS];
	double		Sa[NPERS];

/* ***************************************************************************** */
	bperj[0]  = 0.1;
	bperj[1]  = 0.15;
	bperj[2]  = 0.2;
	bperj[3]  = 0.3;
	bperj[4]  = 0.4;
	bperj[5]  = 0.5;
	bperj[6]  = 0.75;
	bperj[7]  = 1.0;
	bperj[8]  = 1.5;
	bperj[9]  = 2.0;
	bperj[10] = 3.0;
/* ***************************************************************************** */
	bper[ 1 - 1] = 0.02;
	bper[ 2 - 1] = 0.02;
	bper[ 3 - 1] = 0.05;
	bper[ 4 - 1] = 0.10;
	bper[ 5 - 1] = 0.20;
	bper[ 6 - 1] = 0.04;
	bper[ 7 - 1] = 0.04;
	bper[ 8 - 1] = 0.04;
	bper[ 9 - 1] = 0.05;
	bper[10 - 1] = 0.05;
	bper[11 - 1] = 0.05;
	bper[12 - 1] = 0.06;
	bper[13 - 1] = 0.06;
	bper[14 - 1] = 0.07;
	bper[15 - 1] = 0.07;
	bper[16 - 1] = 0.07;
	bper[17 - 1] = 0.08;
	bper[18 - 1] = 0.09;
	bper[19 - 1] = 0.09;
	bper[20 - 1] = 0.10;
	bper[21 - 1] = 0.10;
	bper[22 - 1] = 0.11;
	bper[23 - 1] = 0.12;
	bper[24 - 1] = 0.13;
	bper[25 - 1] = 0.14;
	bper[26 - 1] = 0.15;
	bper[27 - 1] = 0.16;
	bper[28 - 1] = 0.17;
	bper[29 - 1] = 0.18;
	bper[30 - 1] = 0.19;
	bper[31 - 1] = 0.20;
	bper[32 - 1] = 0.22;
	bper[33 - 1] = 0.24;
	bper[34 - 1] = 0.26;
	bper[35 - 1] = 0.28;
	bper[36 - 1] = 0.30;
	bper[37 - 1] = 0.32;
	bper[38 - 1] = 0.34;
	bper[39 - 1] = 0.36;
	bper[40 - 1] = 0.38;
	bper[41 - 1] = 0.40;
	bper[42 - 1] = 0.42;
	bper[43 - 1] = 0.44;
	bper[44 - 1] = 0.46;
	bper[45 - 1] = 0.48;
	bper[46 - 1] = 0.50;
	bper[47 - 1] = 0.55;
	bper[48 - 1] = 0.60;
	bper[49 - 1] = 0.65;
	bper[50 - 1] = 0.70;
	bper[51 - 1] = 0.75;
	bper[52 - 1] = 0.80;
	bper[53 - 1] = 0.85;
	bper[54 - 1] = 0.90;
	bper[55 - 1] = 0.95;
	bper[56 - 1] = 1.00;
	bper[57 - 1] = 1.10;
	bper[58 - 1] = 1.20;
	bper[59 - 1] = 1.30;
	bper[60 - 1] = 1.40;
	bper[61 - 1] = 1.50;
	bper[62 - 1] = 1.60;
	bper[63 - 1] = 1.70;
	bper[64 - 1] = 1.80;
	bper[65 - 1] = 1.90;
	bper[66 - 1] = 2.00;
	bper[67 - 1] = 2.20;
	bper[68 - 1] = 2.40;
	bper[69 - 1] = 2.60;
	bper[70 - 1] = 2.80;
	bper[71 - 1] = 3.00;
	bper[72 - 1] = 3.20;
	bper[73 - 1] = 3.40;
	bper[74 - 1] = 3.60;
	bper[75 - 1] = 3.80;
	bper[76 - 1] = 4.00;
	bper[77 - 1] = 4.20;
	bper[78 - 1] = 4.40;
	bper[79 - 1] = 4.60;
	bper[80 - 1] = 4.80;
	bper[81 - 1] = 5.00;
	bper[82 - 1] = 5.50;
	bper[83 - 1] = 6.00;
	bper[84 - 1] = 6.50;
	bper[85 - 1] = 7.00;
	bper[86 - 1] = 7.50;
	bper[87 - 1] = 8.00;
	bper[88 - 1] = 8.50;
	bper[89 - 1] = 9.00;
	bper[90 - 1] = 9.50;
	bper[91 - 1] =10.00;
	bper[92 - 1] =11.00;
	bper[93 - 1] =12.00;
	bper[94 - 1] =13.00;
	bper[95 - 1] =14.00;
	bper[96 - 1] =15.00;
/* ***************************************************************************** */

	/* Get arguments */
	if (argc != 4 && argc != 5)
		exit (-1);
	damp = atof(argv[1]);
	sps = atof(argv[2]);
	scale = atof(argv[3]);
	if (argc == 5 && streq("-j", argv[4])) {
		nper = NPERSj;
		per = bperj;
	}
	else {
		nper = NPERS;
		per = bper;
	}

	/* Initialize */
	hh = damp / (double)100.;
	dt = (double)1.0 / sps;

	/* read accelerogram */
	nn = 0;
	while (NULL != fgets(inbuf, (int)128, stdin)) {
		if (1 != sscanf(inbuf, "%lf", &(aa[nn])))
			exit (-2);

		if (nn > NPTS) {
			printf("Trace length exceeds NPTS = %d\n", NPTS);
		}

		aa[nn] = scale * aa[nn];
		nn++;
	}

	/* Run rsc() across accelerograms */
	(void)printf("%.1lf%% damping; %.1lf sps; scaled *%lf to get cm/s/s:\n",
		damp, sps, scale);

	for (ii = 0 ; ii < nper ; ii++) {
		ww      = 2.0 * PI / per[ii];
		result  = rsc(hh, ww, dt, aa, nn);

		Sd[ii]  = result.dd;
		PSV[ii] = result.pv;
		Sv[ii]  = result.vv;
		Sa[ii]  = result.zz;

#if 0
                for (jj=0 ; jj < nn ; jj++)
                     printf("%lf %lf %lf\n",
                          xx[jj], vv[jj], zz[jj]);
#endif
	}

	write_vec("per", per, nper);
	write_vec("Sd", Sd, nper);
	write_vec("PSV", PSV, nper);
	write_vec("Sv", Sv, nper);
	write_vec("Sa", Sa, nper);

	return(0);
}

void
write_vec(name, vector, nn) 
	char	*name;						/* (MatLab) vector name			*/
	double	vector[];					/* data to write out			*/
	int	nn;								/* number of values to write	*/
{
	int	ii;

	if (nn < 1)
		return;

	printf("\n%s = [%.10lf\n", name, vector[0]);
	if (nn > 2) {
		for(ii = 1 ; ii < nn - 1 ; ii++) {
			printf("%.10lf\n", vector[ii]);
		}
	}
	printf("%.10lf];\n\n", vector[nn - 1]);
}

